@@ -29,6 +29,7 @@
 #include <errno.h>
 #include <string.h>
 #include <inttypes.h>
+#include <stdlib.h>
 
 #include "../SEAP/generic/rbt/rbt.h"
 #include "probe-api.h"
@@ -84,6 +85,80 @@ static void probe_icache_item_setID(SEXP_t *item, SEXP_ID_t item_ID)
         return;
 }
 
+static int icache_lookup(rbt_t *tree, int64_t item_id, probe_iqpair_t *pair) {
+
+	probe_citem_t *cached = NULL;
+
+	if (rbt_i64_get(tree, item_id, (void**)&cached) != 0) {
+		return -1;
+	}
+
+	/*
+	* Maybe a cache HIT
+	*/
+	dI("cache HIT #1");
+
+	register uint16_t i;
+	for (i = 0; i < cached->count; ++i) {
+		SEXP_t rest1;
+		SEXP_t* rest_r1 = SEXP_list_rest_r(&rest1, pair->p.item);
+
+		SEXP_t rest2;
+		SEXP_t* rest_r2 = SEXP_list_rest_r(&rest2, cached->item[i]);
+
+		if (SEXP_deepcmp(rest_r1, rest_r2)) {
+			SEXP_free_r(&rest1);
+			SEXP_free_r(&rest2);
+			break;
+		}
+
+		SEXP_free_r(&rest1);
+		SEXP_free_r(&rest2);
+	}
+
+	if (i == cached->count) {
+		/*
+		* Cache MISS
+		*/
+		dI("cache MISS");
+
+		cached->item = oscap_realloc(cached->item, sizeof(SEXP_t *) * ++cached->count);
+		cached->item[cached->count - 1] = pair->p.item;
+
+		/* Assign an unique item ID */
+		probe_icache_item_setID(pair->p.item, item_id);
+	} else {
+		/*
+		* Cache HIT
+		*/
+		dI("cache HIT #2 -> real HIT");
+		SEXP_free(pair->p.item);
+		pair->p.item = cached->item[i];
+	}
+	return 0;
+}
+
+static void icache_add_to_tree(rbt_t *tree, int64_t item_id, probe_iqpair_t *pair) {
+
+	probe_citem_t *cached = oscap_talloc(probe_citem_t);
+	cached->item = oscap_talloc(SEXP_t *);
+	cached->item[0] = pair->p.item;
+	cached->count = 1;
+
+	/* Assign an unique item ID */
+	probe_icache_item_setID(pair->p.item, item_id);
+
+	if (rbt_i64_add(tree, (int64_t)item_id, (void **)cached, NULL) != 0) {
+		dE("Can't add item (k=%"PRIi64" to the cache (%p)", item_id, tree);
+
+		oscap_free(cached->item);
+		oscap_free(cached);
+
+		/* now what? */
+		abort();
+	}
+}
+
 static void *probe_icache_worker(void *arg)
 {
         probe_icache_t *cache = (probe_icache_t *)(arg);
@@ -119,8 +194,8 @@ static void *probe_icache_worker(void *arg)
 
         while(pthread_cond_wait(&cache->queue_notempty, &cache->queue_mutex) == 0) {
                 assume_d(cache->queue_cnt > 0, NULL);
-        next:
-                dD("Extracting item from the cache queue: cnt=%"PRIu16", beg=%"PRIu16"", cache->queue_cnt, cache->queue_beg);
+        do {
+                dI("Extracting item from the cache queue: cnt=%"PRIu16", beg=%"PRIu16"", cache->queue_cnt, cache->queue_beg);
                 /*
                  * Extract an item from the queue and update queue beg, end & cnt
                  */
@@ -138,14 +213,6 @@ static void *probe_icache_worker(void *arg)
 			 cache->queue_end == cache->queue_beg :
 			 cache->queue_end != cache->queue_beg, NULL);
 
-                /*
-                 * Release the mutex
-                 */
-                if (pthread_mutex_unlock(&cache->queue_mutex) != 0) {
-                        dE("An error ocured while unlocking the queue mutex: %u, %s",
-                           errno, strerror(errno));
-                        abort();
-                }
 
                 dD("Signaling `notfull'");
 
@@ -154,6 +221,14 @@ static void *probe_icache_worker(void *arg)
                            errno, strerror(errno));
                         abort();
                 }
+                /*
+                 * Release the mutex
+                 */
+                if (pthread_mutex_unlock(&cache->queue_mutex) != 0) {
+                        dE("An error ocured while unlocking the queue mutex: %u, %s",
+                           errno, strerror(errno));
+                        abort();
+                }
 
                 if (pair->cobj == NULL) {
                         /*
@@ -169,81 +244,23 @@ static void *probe_icache_worker(void *arg)
                                 abort();
                         }
                 } else {
-                        probe_citem_t *cached = NULL;
 
                         dD("Handling cache request");
 
                         /*
                          * Compute item ID
                          */
+			dD("pair address: %"PRIu64, (uint64_t) pair);
+			dD("item address: %"PRIu64, (uint64_t) pair->p.item);
                         item_ID = SEXP_ID_v(pair->p.item);
                         dD("item ID=%"PRIu64"", item_ID);
 
-                        /*
-                         * Perform cache lookup
-                         */
-                        if (rbt_i64_get(cache->tree, (int64_t)item_ID, (void *)&cached) == 0) {
-                                register uint16_t i;
-				SEXP_t   rest1, rest2;
-                                /*
-                                 * Maybe a cache HIT
-                                 */
-                                dD("cache HIT #1");
-
-                                for (i = 0; i < cached->count; ++i) {
-                                        if (SEXP_deepcmp(SEXP_list_rest_r(&rest1, pair->p.item),
-							 SEXP_list_rest_r(&rest2, cached->item[i])))
-					{
-						SEXP_free_r(&rest1);
-						SEXP_free_r(&rest2);
-                                                break;
-					}
-
-					SEXP_free_r(&rest1);
-					SEXP_free_r(&rest2);
-                                }
-
-                                if (i == cached->count) {
-                                        /*
-                                         * Cache MISS
-                                         */
-                                        dD("cache MISS");
-
-                                        cached->item = oscap_realloc(cached->item, sizeof(SEXP_t *) * ++cached->count);
-                                        cached->item[cached->count - 1] = pair->p.item;
-
-                                        /* Assign an unique item ID */
-                                        probe_icache_item_setID(pair->p.item, item_ID);
-                                } else {
-                                        /*
-                                         * Cache HIT
-                                         */
-                                        dD("cache HIT #2 -> real HIT");
-                                        SEXP_free(pair->p.item);
-                                        pair->p.item = cached->item[i];
-                                }
-                        } else {
+                        if (icache_lookup(cache->tree, item_ID, pair) != 0) {
                                 /*
                                  * Cache MISS
                                  */
-                                dD("cache MISS");
-                                cached = oscap_talloc(probe_citem_t);
-                                cached->item = oscap_talloc(SEXP_t *);
-                                cached->item[0] = pair->p.item;
-                                cached->count = 1;
-
-                                /* Assign an unique item ID */
-                                probe_icache_item_setID(pair->p.item, item_ID);
-
-                                if (rbt_i64_add(cache->tree, (int64_t)item_ID, (void *)cached, NULL) != 0) {
-                                        dE("Can't add item (k=%"PRIi64" to the cache (%p)", (int64_t)item_ID, cache->tree);
-
-                                        oscap_free(cached->item);
-                                        oscap_free(cached);
-
-                                        /* now what? */
-                                        abort();
-                                }
+                                dI("cache MISS");
+                                icache_add_to_tree(cache->tree, item_ID, pair);
                         }
 
                         if (probe_cobj_add_item(pair->cobj, pair->p.item) != 0) {
@@ -257,8 +274,7 @@ static void *probe_icache_worker(void *arg)
                         abort();
                 }
 
-                if (cache->queue_cnt > 0)
-                        goto next;
+                } while (cache->queue_cnt > 0);
         }
 
         return (NULL);
@@ -363,6 +379,12 @@ int probe_icache_add(probe_icache_t *cache, SEXP_t *cobj, SEXP_t *item)
 
         ret = __probe_icache_add_nolock(cache, cobj, item, NULL);
 
+        if (pthread_cond_signal(&cache->queue_notempty) != 0) {
+                dE("An error ocured while signaling the `notempty' condition: %u, %s",
+                   errno, strerror(errno));
+                return (-1);
+        }
+
         if (pthread_mutex_unlock(&cache->queue_mutex) != 0) {
                 dE("An error ocured while unlocking the queue mutex: %u, %s",
                    errno, strerror(errno));
@@ -372,12 +394,6 @@ int probe_icache_add(probe_icache_t *cache, SEXP_t *cobj, SEXP_t *item)
         if (ret != 0)
                 return (-1);
 
-        if (pthread_cond_signal(&cache->queue_notempty) != 0) {
-                dE("An error ocured while signaling the `notempty' condition: %u, %s",
-                   errno, strerror(errno));
-                return (-1);
-        }
-
         return (0);
 }
 
@@ -555,10 +571,9 @@ static void probe_icache_free_node(struct rbt_i64_node *n)
 {
         probe_citem_t *ci = (probe_citem_t *)n->data;
 
-        while (ci->count > 0) {
-                SEXP_free(ci->item[ci->count - 1]);
-                --ci->count;
-        }
+	for ( ; ci->count > 0 ; --ci->count ) {
+		SEXP_free(ci->item[ci->count - 1]);
+	}
 
         oscap_free(ci->item);
         oscap_free(ci);
