@@ -76,6 +76,21 @@ struct rpmverify_res {
 #define RPMVERIFY_SKIP_GHOST  0x2000000000000000
 #define RPMVERIFY_RPMATTRMASK 0x00000000ffffffff
 
+/* In rmplib older than 4.7 some of the enum values aren't defined.
+ * We need to provide fallback definitions.
+ */
+#ifndef HAVE_RPM47
+	/* *VERIFY_FILEDIGEST were introduced as aliases to *VERIFY_MD5
+	 * They all have the same value (1) - see 'rpm/rpmvf.h'.
+	 */
+	#define RPMVERIFY_FILEDIGEST RPMVERIFY_MD5
+	#define VERIFY_FILEDIGEST VERIFY_MD5
+	/* VERIFY_CAPS is not supported in older rpmlib.
+	 * We can set it to 0 because 0 is neutral to bit OR operation.
+	 */
+	#define VERIFY_CAPS 0
+#endif
+
 static struct rpm_probe_global g_rpm;
 
 #define RPMVERIFY_LOCK   RPM_MUTEX_LOCK(&g_rpm.mutex)
@@ -297,7 +312,9 @@ void probe_preload ()
 void *probe_init (void)
 {
 	probe_setoption(PROBEOPT_OFFLINE_MODE_SUPPORTED, PROBE_OFFLINE_CHROOT);
+#ifdef HAVE_RPM46
 	rpmlogSetCallback(rpmErrorCb, NULL);
+#endif
 	if (rpmReadConfigFiles ((const char *)NULL, (const char *)NULL) != 0) {
 		dI("rpmReadConfigFiles failed: %u, %s.", errno, strerror (errno));
 		return (NULL);
@@ -418,6 +435,10 @@ int probe_main (probe_ctx *ctx, void *arg)
 	uint64_t collect_flags = 0;
 	unsigned int i;
 
+	if (arg == NULL) {
+		return PROBE_EINIT;
+	}
+
 	if (g_rpm.rpmts == NULL) {
 		probe_cobj_set_flag(probe_ctx_getresult(ctx), SYSCHAR_FLAG_NOT_APPLICABLE);
 		return 0;
