@@ -84,6 +84,19 @@ struct oscap_source *oscap_source_new_from_file(const char *filepath)
 	return source;
 }
 
+struct oscap_source *oscap_source_clone(struct oscap_source *old)
+{
+	struct oscap_source *new = (struct oscap_source *) oscap_calloc(1, sizeof(struct oscap_source));
+	new->scap_type = old->scap_type;
+	new->origin.type = old->origin.type;
+	new->origin.version = oscap_strdup(old->origin.version);
+	new->origin.filepath = oscap_strdup(old->origin.filepath);
+	new->origin.memory = oscap_strdup(old->origin.memory);
+	new->origin.memory_size = old->origin.memory_size;
+	new->xml.doc = xmlCopyDoc(old->xml.doc, true);
+	return new;
+}
+
 /**
  * Allocate oscap_source struct and fill for memory data
  * @param size_t size Size of data
@@ -178,6 +191,11 @@ oscap_document_type_t oscap_source_get_scap_type(struct oscap_source *source)
 	return source->scap_type;
 }
 
+const char *oscap_source_get_filepath(struct oscap_source *source)
+{
+	return source->origin.filepath;
+}
+
 static void xmlErrorCb(struct oscap_string *buffer, const char * format, ...)
 {
 	va_list ap;
@@ -237,11 +255,13 @@ xmlDoc *oscap_source_get_xmlDoc(struct oscap_source *source)
 
 	if (source->xml.doc == NULL) {
 		if (source->origin.memory != NULL) {
-#ifdef HAVE_BZ2
 			if (bz2_memory_is_bzip(source->origin.memory, source->origin.memory_size)) {
+#ifdef HAVE_BZ2
 				source->xml.doc = bz2_mem_read_doc(source->origin.memory, source->origin.memory_size);
-			} else
+#else
+				oscap_seterr(OSCAP_EFAMILY_OSCAP, "Unable to unpack bz2 from buffer memory '%s'. Please compile OpenSCAP with bz2 support.", oscap_source_readable_origin(source));
 #endif
+			} else
 			{
 				source->xml.doc = xmlReadMemory(source->origin.memory, source->origin.memory_size, NULL, NULL, 0);
 				if (source->xml.doc == NULL) {
@@ -263,11 +283,14 @@ xmlDoc *oscap_source_get_xmlDoc(struct oscap_source *source)
 				source->xml.doc = NULL;
 				oscap_seterr(OSCAP_EFAMILY_GLIBC, "Unable to open file: '%s'", oscap_source_readable_origin(source));
 			} else {
-#ifdef HAVE_BZ2
 				if (bz2_fd_is_bzip(fd)) {
+#ifdef HAVE_BZ2
 					source->xml.doc = bz2_fd_read_doc(fd);
-				} else
+#else
+					source->xml.doc = NULL;
+					oscap_seterr(OSCAP_EFAMILY_OSCAP, "Unable to unpack bz2 file '%s'. Please compile OpenSCAP with bz2 support.", oscap_source_readable_origin(source));
 #endif
+				} else
 				{
 					source->xml.doc = xmlReadFd(fd, NULL, NULL, 0);
 					if (source->xml.doc == NULL) {
@@ -308,6 +331,9 @@ int oscap_source_validate(struct oscap_source *source, xml_reporter reporter, vo
 		ret = -1;
 	} else {
 		const char *schema_version = oscap_source_get_schema_version(source);
+		if (!schema_version) {
+			schema_version = "unknown schema version";
+		}
 		const char *type_name = oscap_document_type_to_string(scap_type);
 		const char *origin = oscap_source_readable_origin(source);
 		dD("Validating %s (%s) document from %s.", type_name, schema_version, origin);
